# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is ManagedMeetings version 1.0.
#
# The Initial Owner of the Original Code is European Environment
# Agency (EEA).  Portions created by CMG for EEA are
# Copyright (C) European Environment Agency.  All
# Rights Reserved.
#
# Contributor(s):
# Tomas Hjelmberg, CMG
#

##########################################
# <ZOPEDIR>/lib/python/products/Attendee/Attendee.py

from OFS import SimpleItem, Image
from Globals import DTMLFile, MessageDialog
from AccessControl import getSecurityManager, Permissions
from string import *
import re

class Attendee(SimpleItem.SimpleItem):

    """
    Course Attendee class
    """

    meta_type = 'Meeting Attendee'

    manage_options = (
        {'label': 'Properties', 'action': 'manage_editForm',},
        {'label': 'View', 'action': 'index_html',},
    )

    __ac_permissions__=(
        ('Register for Meetings', ('manage_addAttendee',),('Manager',)),
        ('Change Meetings',       ('manage_editAction','manage_editForm'),('Manager',)),
        ('View',                  ('index_html',),('Manager','Anonymous')),
        )

    def __setstate__(self,state):
        Attendee.inheritedAttribute("__setstate__") (self, state)
        if not hasattr(self, "resume"): # backwards compatibility
            self.resume = ''

    def manage_editAction(self, sn, givenname, organisation, mail,
            tel, postaladdress, attending, resume='', role='', verified=0, REQUEST=None):
        "Changes the product values"
        self.title = givenname + ' ' + sn
        self.sn = sn
        self.givenname = givenname
        self.organisation = organisation
        self.mail = mail
        self.tel = tel
        self.postaladdress = postaladdress
        self.attending = attending
        self.role = role
        self.resume = resume
        self.verified = verified
	if REQUEST is not None:
	    return MessageDialog(
		title = 'Edited',
		message = "The Attendee %s has been changed!" % self.id,
		action = 'manage_main',
		)

    index_html = DTMLFile('www/AttendeeIndex', globals())

    manage_editForm = DTMLFile('www/AttendeeEditForm', globals())

    # constructor pages. Only used when the product is added to a folder.

def manage_addAttendee(self,id , sn, givenname, organisation, mail,
        tel, postaladdress, attending, resume='', role='', verified=0, REQUEST=None):
    "Add an attendee to a folder."

    title = givenname + ' ' + sn
    ob=Attendee()
    ob.id=id
    ob.title = title
    ob.sn = sn
    ob.givenname = givenname
    ob.organisation = organisation
    ob.mail = mail
    ob.tel = tel
    ob.postaladdress = postaladdress
    ob.attending = attending
    ob.role = role
    ob.resume = resume
    ob.verified = verified
    self._setObject(id, ob)
    ob=self._getOb(id)
    if REQUEST is not None:
        return MessageDialog(
            title = 'Created',
            message = "The Attendee %s has been created!" % ob.id,
            action = 'manage_main?update_menu=1',
            )

manage_addattendeeForm = DTMLFile('www/AttendeeAddForm', globals())
