# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is ManagedMeetings version 1.0.
#
# The Initial Owner of the Original Code is European Environment
# Agency (EEA).  Portions created by CMG for EEA are
# Copyright (C) European Environment Agency.  All
# Rights Reserved.
#
# Contributor(s):
# Tomas Hjelmberg, CMG
# Soren Roug, EEA
__doc__ = """
              ManagedMeetings product module.
              The ManagedMeetings-Product is a meeting-scheduler/planner

$Id: ManagedMeetings.py,v 1.21 2002/05/21 21:18:59 roug Exp $
"""
__version__='$Revision: 1.21 $'[10:-2]


from OFS import Folder,Image
from Globals import DTMLFile, MessageDialog
from AccessControl import getSecurityManager, Permissions

import Products
import os
from os.path import join
import re
import string
import tempfile
import types
import Globals
import StringIO

manage_addManagedMeetingsForm=DTMLFile('www/ManagedMeetings_addForm', globals())

def manage_addManagedMeetings(self, id, title='', description='',
	event_types=[], REQUEST=None):
    """Add a new ManagedMeetings object with id=title."""

    ob=ManagedMeetings(id, title, description, event_types)
    ob.id = id
    self._setObject(id, ob)

    indexfile = open(join(Globals.SOFTWARE_HOME ,
        'Products','ManagedMeetings','www','ManagedMeetingsIndex.dtml'))
    content = indexfile.read()
    indexfile.close()
    ob.manage_addDTMLMethod('index_html', title='Current/Future Events', file=content)

    indexfile = open(join(Globals.SOFTWARE_HOME ,
        'Products','ManagedMeetings','www','ManagedMeetingsPast.dtml'))
    content = indexfile.read()
    indexfile.close()
    ob.manage_addDTMLMethod('past_html', title='Past Events', file=content)

    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)

class ManagedMeetings(Folder.Folder):
    """A Folder-like product that can contain Attendees and course-material"""

    meta_type="Folder for Meetings"

    # what properties have we?
    _properties=(
        {'id':'title',		'type':'string'},
        {'id':'description', 	'type':'text'},
        {'id':'event_types', 	'type':'lines'},
    )

    __ac_permissions__=(
        (Permissions.view_management_screens, ('manage_uploadForm',)),
        ('View',                              ('open_rss','index_html')),
        ('Change Meetings folders',            ('ManagedMeetingsEdit',))
	)

    ################################
    # Public methods               #
    ################################

    def __str__(self): return self.index_html()
    def __len__(self): return 1

    def all_meta_types(self):
	""" What can you put inside me? """
	f = lambda x: x['name'] in ('DTML Method', 'DTML Document',
	    'Page Template', 'Meeting Location', 'Meeting', 'File')
	return filter(f, Products.meta_types)

    def manage_addObject(self):
	"""Add a new object to the Meeting"""
	ob=self._getOb(id)

	checkPermission=getSecurityManager().checkPermission

        if file is not None:
	    if not checkPermission('Add Documents, Images, and Files', ob):
		raise 'Unauthorized', (
		    'You are not authorized to add DTML Documents.'
		    )
	    return ob.manage_upload(file,subfolders,REQUEST=REQUEST)

        if REQUEST is not None:
            return MessageDialog(title = 'Created',
		     message = 'The ManagedMeetings %s was successfully created!' % id,
		     action = 'manage_main',)

   ## manage_options=(Folder.Folder.manage_options+
##                    ({ "label": "ManagedMeetings",
##                       "action": "MeetingEdit"},)
##                    )
#   manage_addLocationForm=DTMLFile('www/Location_addForm', globals())
#   manage_addMeetingForm=DTMLFile('www/Meeting_addForm', globals())
#   location_add = DTMLFile('www/location_add', globals()) # Add a location
#   meeting_add = DTMLFile('www/meeting_add', globals()) # Add a meeting

    open_rss = DTMLFile('www/openrdf', globals())

    ################################
    # Init method                  #
    ################################
	
    def __init__(self, id, title='', description='',event_types=[], REQUEST=None):
	""" initialize a new instance of ManagedMeetings """
	self.id = id
	self.title = title
	self.description = description
	self.event_types = event_types

    def __setstate__(self,state):
	ManagedMeetings.inheritedAttribute("__setstate__") (self, state)
	if not hasattr(self, "event_types"): # backwards compatibility
	    self.event_types = []

    ################################
    # Protected management methods #
    ################################
	
    def manage_editManagedMeetings(self, title='', description='', event_types=[], REQUEST=None):
        """ Manage the edited values """

	self.title = title
	self.description = description
	self.event_types = event_types

	# update ZCatalog
	# self.reindex_object()
	if REQUEST is not None:
	    return MessageDialog(
		title = 'Edited',
		message = "The properties of %s have been changed!" % self.id,
		action = './manage_main',
		)


Globals.default__class_init__(ManagedMeetings)
