# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is ManagedMeetings version 1.0.
#
# The Initial Owner of the Original Code is European Environment
# Agency (EEA).  Portions created by CMG for EEA are
# Copyright (C) European Environment Agency.  All
# Rights Reserved.
#
# Contributor(s):
# Tomas Hjelmberg, CMG
# Soren Roug, EEA

from OFS import SimpleItem, ObjectManager, FindSupport
import webdav.Collection
from webdav.WriteLockInterface import WriteLockInterface
from Globals import DTMLFile, MessageDialog
from AccessControl import getSecurityManager, Permissions

import os,re,string
from os.path import join
from time import *
import tempfile
import types
import Globals
import StringIO
import cStringIO

import Products

from Persistence import Persistent # for setstate backwards compatibility

#
# QuotedPrintable functions
#
ESCAPE = '='
MAXLINESIZE = 76
HEX = '0123456789ABCDEF'

def _needsquoting(c):
    """Decide whether a particular character needs to be quoted."""

    return c == ESCAPE or not(' ' < c <= '~')

def _quote(c):
    """Quote a single character."""
    i = ord(c)
    if i == 13:
	return ""
    if i == 10:    # Handle UNIX newline
	return ESCAPE + '0D' + ESCAPE + '0A'
    else:
	return ESCAPE + HEX[i/16] + HEX[i%16]

def _qp_encode(line,offset):
    """Read 'line', apply quoted-printable encoding, and return it.
    """
    output=''
    if not line:
	return None
    new = ''
    prev = ''
    for c in line:
	if _needsquoting(c):
	    c = _quote(c)
	if offset + len(new) + len(c) >= MAXLINESIZE:
	    output = output + new + ESCAPE + '\r\n '
	    offset = 0
	    new = ''
	new = new + c
	prev = c
    if prev in (' ', '\t'):
	output = output + new + ESCAPE + '\r\n'
    else:
	output = output + new + '\r\n'
    return output

def _slash_encode(line,offset):
    "Backslash-escape a string"
    new = ''
    output=''
    for c in line:
	x = c
	if c in [';','\\', ',' ]:
	    x = '\\' + c
	if c == '\n':
	    x = '\\n'
	if offset + len(new) + len(c) >= MAXLINESIZE:
	    output = output + new + '\r\n '
	    offset = 0
	    new = ''
	new = new + x
    output = output + new + '\r\n'
    return output


manage_addMeetingForm=DTMLFile('www/Meeting_addForm', globals())

def manage_addMeeting(self, id, title, event_type, location,
        txtlocation, description, startdate, enddate,
	organiser, organiser_email, max_attendees,
	agenda_url='',minutes_url='',
	allow_registrations=0, allow_anonymous=0, active=0, REQUEST=None):
    """Add a new Meeting object"""
    ob=Meeting()
    ob.id=id
    ob.title = title
    ob.event_type = event_type
    ob.location = location
    ob.txtlocation=txtlocation
    ob.description = description
    ob.startdate = startdate
    ob.enddate = enddate
    ob.agenda_url = agenda_url
    ob.minutes_url = minutes_url
    ob.organiser = organiser
    ob.organiser_email = organiser_email
    ob.max_attendees = max_attendees
    ob.allow_registrations = allow_registrations
    ob.allow_anonymous = allow_anonymous
    ob.active = active
    ob.generation = 0
    self._setObject(id, ob)
    ob=self._getOb(id)

    indexfile = open(join(Globals.SOFTWARE_HOME ,
	'Products','ManagedMeetings','www','MeetingIndex.dtml'))
    content = indexfile.read()
    indexfile.close()

    ob.manage_addDTMLMethod('index_html', title='Default View', file=content)


    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)

class Meeting(
    ObjectManager.ObjectManager,
    webdav.Collection.Collection,
    SimpleItem.SimpleItem,
    FindSupport.FindSupport,
    ):
    """A Folder-like product that can contain Attendees and course-material"""

    meta_type="Meeting"

    __ac_permissions__=(
#       (Permissions.view_management_screens, ('manage_uploadForm',)),
        ('Register for Meetings', ('signup',
				   'manage_addAnonAttendeeAction'),('Manager',)),
        ('Authenticated Registration', ('authenticatedsignup',
				   'manage_addAttendeeAction'),('Manager',)),
        ('Change Meetings',       ('manage_propertiesForm',),('Manager',)),
	('Use mailhost services', ('send_email',),),
	('View Attendees', ('attendees',)),
        ('View',                  ('agenda', 'tracks','daybyday',
			'send_email','showlocation'),),
	)

    ################################
    # Public methods               #
    ################################

    def __str__(self): return self.index_html()
    def __len__(self): return 1

    def createhtmlmail (self, text, subject, filein, sender):
	"""Create a mime-message that will render HTML in popular
	MUAs, text in better ones"""
	import MimeWriter
	import mimetools
	import cStringIO

	out = cStringIO.StringIO() # output buffer for our message
	txtin = cStringIO.StringIO(text)

	writer = MimeWriter.MimeWriter(out)
	#
	# set up some basic headers... we put subject here
	# because smtplib.sendmail expects it to be in the
	# message body
	#
	writer.addheader("To", sender)
	writer.addheader("Subject", subject)
	writer.addheader("MIME-Version", "1.0")
	#
	# start the multipart section of the message
	# multipart/alternative seems to work better
	# on some MUAs than multipart/mixed
	#
	writer.startmultipartbody("mixed")
	writer.flushheaders()
	#
	# the plain text section
	#
	subpart = writer.nextpart()
	subpart.addheader("Content-Transfer-Encoding", "quoted-printable")
	pout = subpart.startbody("text/plain", [("charset", 'iso-8859-1')])
	mimetools.encode(txtin, pout, 'quoted-printable')
	txtin.close()
	#
	# start the attach subpart of the message
	#
	#context = self.getContext()
	#meetingfolder = context[id]

	for t in self.objectValues(['File', 'DTML Document', 'ExtFile', 'URL Reference',]):
	    if (filein.count(t.getId())):
		fil = cStringIO.StringIO(str(t))
		subpart = writer.nextpart()
		subpart.addheader("Content-Transfer-Encoding", "base64")
		#
		# returns us a file-ish object we can write to
		#
		pout = subpart.startbody("application/octet-stream", [('name', t.getId())])
		mimetools.encode(fil, pout, 'base64')
		fil.close()
	#
	# Now that we're done, close our writer and
	# return the message body
	#
	writer.lastpart()
	msg = out.getvalue()
	out.close()
	return msg

    def send_email(self, subject, comments, attached=[], REQUEST=None):
	""" Sends an email """
	userid = REQUEST.AUTHENTICATED_USER.getUserName()
	reciplist = []
	for t in self.objectValues('Meeting Attendee'):
	    if t.mail != '':
		reciplist.append('<' + t.mail + '>')
	recipients = string.join(reciplist,',\n  ') #Unused

	sendermail = 'MeetingManager@' + REQUEST['SERVER_NAME']
	if hasattr(self, 'organiser_email'):
	     if getattr(self, 'organiser_email') != '':
		sendermail = self.organiser_email
	sender = userid + ' <' + sendermail + '>'

	import smtplib		
	message = self.createhtmlmail( comments, subject, attached, sender)
	server = smtplib.SMTP("localhost")
	server.sendmail(sender, reciplist, message)
	server.quit()
	if REQUEST is not None:
	    return MessageDialog(
		title = 'Mailed',
		message = "The mail %s has been sent!" % self.id,
		action = './manage_main',
		)

    def all_meta_types(self):
        """ What can you put inside me? """
        f = lambda x: x['name'] in (
		'DTML Method', 'DTML Document', 'Page Template',
		'Image', 'File',
		'Meeting Attendee', 'Meeting Session', 'Meeting Agendaitem',
		'Meeting Location', 'Meeting Track','Meeting Break')
        return filter(f, Products.meta_types)

    def manage_addObject(self):
        """Add a new object to the Meeting"""
        #ob=self._getOb(id)

        checkPermission=getSecurityManager().checkPermission

        if file is not None:
            if not checkPermission('Add Documents, Images, and Files', ob):
                raise 'Unauthorized', (
                    'You are not authorized to add DTML Documents.'
                    )
            return ob.manage_upload(file,subfolders,REQUEST=REQUEST)

        if REQUEST is not None:
            return MessageDialog(title = 'Created',
                                 message = 'The Meeting %s was successfully created!' % id,
                                 action = 'manage_main?update_menu=1',)

    manage_options=(
        (ObjectManager.ObjectManager.manage_options[0],)+
        (
        {'label':'Properties', 'action':'manage_propertiesForm',
         'help':('ManagedMeetings','Meeting_Edit.stx')},
        {'label':'View', 'action':'index_html', },
        )+
        SimpleItem.SimpleItem.manage_options+
        FindSupport.FindSupport.manage_options
        )

    signup_add=DTMLFile('www/authenticated_add', globals())

    signup=DTMLFile('www/MeetingSignUp', globals())

    # Method to call if someone is logged in and wants to register himself
    authenticatedsignup=DTMLFile('www/authenticatedsignup', globals())

    def manage_addAnonAttendeeAction(self, sn, givenname, organisation, mail,
            tel, postaladdress, attending, REQUEST=None):
	"Create an attendee, by calling the constructor"

	mail = str(mail)
	pat = re.compile(r'([a-zA-Z][\w-]*@[\w-]+(?:\.[\w-]+)*)')
	if pat.search(mail) is None:
	    return MessageDialog(
		title = 'Bad email address',
		message = "Your registration couldn't succeed due to invalid email address",
		action = 'signup',
		)
	id = string.replace(mail, '@', '_')

	self.manage_addProduct['ManagedMeetings'].manage_addAttendee(
	        id , sn, givenname, organisation, mail,
		tel, postaladdress, attending, '', 'Attendee', 0)
	if REQUEST is not None:
	    return MessageDialog(
		title = 'Created',
		message = "Thanks for your registration",
		action = 'index_html',
		)

    def manage_addAttendeeAction(self, sn, givenname, organisation, mail,
            tel, postaladdress, attending, role, REQUEST=None):
	"Create an attendee, by calling the constructor"

	id = REQUEST.AUTHENTICATED_USER.getUserName()

	self.manage_addProduct['ManagedMeetings'].manage_addAttendee(
	        id , sn, givenname, organisation, mail,
		tel, postaladdress, attending, '', role, 1)
	if REQUEST is not None:
	    return MessageDialog(
		title = 'Created',
		message = "Thanks for your registration",
		action = 'index_html',
		)

    def show_vcs(self,REQUEST,RESPONSE):
	"""Generate vCalendar file"""
	RESPONSE.setHeader('Content-Type', 'text/x-vCalendar')
	RESPONSE.setHeader('Content-Disposition', 'attachment; filename="%s.vcs"' % self.id)
	RESPONSE.write("BEGIN:VCALENDAR\r\n")
	RESPONSE.write("PRODID:-//European Environment Agency//Managed Meetings//EN\r\n")
	RESPONSE.write("VERSION:1.0\r\n")
	RESPONSE.write("BEGIN:VEVENT\r\n")
	RESPONSE.write("DTSTART:%s\r\n" % self.startdate.strftime("%Y%m%dT%H%M%S"))
	RESPONSE.write("DTEND:%s\r\n" % self.enddate.strftime("%Y%m%dT%H%M%S"))
	if self.location != '' :
	    RESPONSE.write("LOCATION;ENCODING=8-bit:%s\r\n" % self.location)
	else:
	    RESPONSE.write("LOCATION;ENCODING=8-bit:%s\r\n" % self.txtlocation)
	RESPONSE.write("UID:%s\r\n" % REQUEST.URL0)
	RESPONSE.write("URL:%s\r\n" % REQUEST.URL1)
	RESPONSE.write("SUMMARY:%s\r\n" % self.title)	
	if self.description != '':
	    RESPONSE.write("DESCRIPTION;ENCODING=QUOTED-PRINTABLE:%s" % \
	       _qp_encode(self.description,37))
# Check for relative address.
#	if self.agenda_url:
#	    RESPONSE.write("ATTACH;VALUE=URL:%s\r\n" % self.agenda_url)

	for t in self.objectValues(['File', 'DTML Document','Image']):
	    RESPONSE.write("ATTACH;VALUE=URL:%s\r\n" % t.absolute_url())

#	for a in self.objectValues('Meeting Attendee'):
#	    RESPONSE.write("ATTENDEE;ROLE=%s;STATUS=%s:%s %s <%s>\r\n" % \
#		       (a.role, a.attending, a.givenname, a.sn, a.mail))
	RESPONSE.write("END:VEVENT\r\n")
	RESPONSE.write("END:VCALENDAR\r\n")

    def show_ics(self,REQUEST,RESPONSE):
	"""Generate iCalendar file"""
	RESPONSE.setHeader('Content-Type', 'text/calendar')
	RESPONSE.setHeader('Content-Disposition', 'attachment; filename="%s.ics"' % self.id)
	RESPONSE.write("BEGIN:VCALENDAR\r\n")
	RESPONSE.write("PRODID:-//European Environment Agency//Managed Meetings//EN\r\n")
	RESPONSE.write("VERSION:2.0\r\n")
	RESPONSE.write("METHOD:PUBLISH\r\n")
	RESPONSE.write("BEGIN:VEVENT\r\n")
#	for a in self.objectValues('Meeting Attendee'):
#	    RESPONSE.write('ATTENDEE;ROLE=REQ-PARTICIPANT:MAILTO:%s %s <%s>\r\n' % \
#		       ( a.givenname, a.sn, a.mail))
	RESPONSE.write('ORGANIZER;CN="%s":MAILTO:%s\r\n' % ( self.organiser, self.organiser_email))
	RESPONSE.write("DTSTART:%s\r\n" % self.startdate.strftime("%Y%m%dT%H%M%S"))
	RESPONSE.write("DTEND:%s\r\n" % self.enddate.strftime("%Y%m%dT%H%M%S"))
	if self.location != '' :
	    RESPONSE.write('LOCATION;ALTREP="%s/%s":%s\r\n' % \
	     (REQUEST.URL2, self.location, self.location))
	else:
	    RESPONSE.write("LOCATION:%s\r\n" % self.txtlocation)
	RESPONSE.write("TRANSP:OPAQUE\r\n")
	RESPONSE.write("SEQUENCE:%d\r\n" % self.generation)
 	RESPONSE.write("UID:%s\r\n" % REQUEST.URL0)
 	RESPONSE.write("URL:%s\r\n" % REQUEST.URL1)
	RESPONSE.write("LAST-MODIFIED:%s\r\n" % \
	    self.bobobase_modification_time().toZone('UTC').strftime("%Y%m%dT%H%M%SZ"))

	RESPONSE.write("DTSTAMP:%s\r\n" % \
	    strftime("%Y%m%dT%H%M%SZ",gmtime(time())))
	if self.description != '':
	    RESPONSE.write("DESCRIPTION:%s" % _slash_encode(self.description,12))
 	RESPONSE.write("SUMMARY:%s\r\n" % self.title)	

	RESPONSE.write("PRIORITY:5\r\n")
	RESPONSE.write("CLASS:PUBLIC\r\n")
	RESPONSE.write("END:VEVENT\r\n")
	RESPONSE.write("END:VCALENDAR\r\n")

    agenda= DTMLFile('www/MeetingAgenda', globals()) # the agenda
    SendEmail_form=DTMLFile('www/SendEmail_form', globals())
    attendees = DTMLFile('www/MeetingAttendees', globals())

    tracks= DTMLFile('www/MeetingTracks', globals()) # the tracks
    speakerlist = DTMLFile('www/MeetingSpeakers', globals())
    daybyday = DTMLFile('www/MeetingDayByDay', globals())


    def showlocation(self,REQUEST=None):
	"""Show location with URL"""
	if self.location != '':
	    locobj = self.aq_acquire(self.location)
	    return '<a href="%s">%s</a>' % \
	     ( locobj.absolute_url(), locobj.title )
	else:
	    return self.txtlocation

#   index_html = DTMLFile('www/MeetingIndex', globals())

    ################################
    # Init method                  #
    ################################

    def __setstate__(self,state):
	Meeting.inheritedAttribute("__setstate__") (self, state)
	if not hasattr(self, "allow_anonymous"): # backwards compatibility
	    self.allow_anonymous = 1
	if not hasattr(self, "txtlocation"): # backwards compatibility
	    self.txtlocation = ""
	if not hasattr(self, "agenda_url"): # backwards compatibility
	    self.agenda_url = ""
	if not hasattr(self, "minutes_url"): # backwards compatibility
	    self.minutes_url = ""
	if not hasattr(self, "generation"): # backwards compatibility
	    self.generation = 0

    ################################
    # Protected management methods #
    ################################

    # Management Interface

    manage_propertiesForm= DTMLFile('www/MeetingEdit', globals())

    def manage_editMeeting(self, title='', event_type='', location='',
	    txtlocation='',
            description='', startdate='', enddate='',
	    agenda_url='',minutes_url='',
	    organiser='', organiser_email='', max_attendees=0,
	    allow_registrations=0, allow_anonymous=0, active=0, REQUEST=None):
        """ Change the edited values """
	self.title = title
	self.event_type = event_type
	self.txtlocation = txtlocation
	self.location = location
	self.description = description
	self.startdate = startdate
	self.enddate = enddate
	self.agenda_url = agenda_url
	self.minutes_url = minutes_url
	self.organiser = organiser
	self.organiser_email = organiser_email
	self.max_attendees = max_attendees
	self.allow_registrations = allow_registrations
	self.allow_anonymous = allow_anonymous
	self.active = active
	self.generation = self.generation + 1
	# update ZCatalog
	# self.reindex_object()
	if REQUEST:
	    message="Saved changes."
	    return self.manage_propertiesForm(self,REQUEST,manage_tabs_message=message)

Globals.default__class_init__(Meeting)
